/* -------------------------------------------------------------
This file is part of DOAP (Dave's Own Access Protocol)

DOAP is (C) Copyright 2011 PurplePixie Systems

DOAP is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DOAP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with DOAP.  If not, see www.gnu.org/licenses

For more information see http://www.purplepixie.org/doap
-------------------------------------------------------------- */

function DOAP_Error(code, desc)
{
	this.code = code;
	this.desc = desc;
}

function DOAP_xmlhttp()
{
	this.xmlhttp = null;
	if (window.XMLHttpRequest)
	{
		this.xmlhttp = new XMLHttpRequest;
		if (this.xmlhttp.overrideMimeType) this.xmlhttp.overrideMimeType("text/xml");
	}
	else if (window.ActiveXObject)
	{
		try
		{
			this.xmlhttp = new ActiveXObject("Msxml2.HTTP");
		}
		catch(e)
		{
			try
			{
				this.xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
			}
			catch(e)
			{
				// Give Up Now
			}
		}
	}
	
	if (this.xmlhttp == null)
	{
		var error = new DOAP_Error(100,"Failed to Create XMLHTTP Object");
		throw error;
	}
	else return true;
}

function DOAP_randomstring( len )
{
	if (!len) var len = 32;
	var allowed = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
	var rstring = "";
	
	for (var i=0; i<len; i++)
	{
		var rchar = Math.floor(Math.random() * allowed.length);
		rstring += allowed.substring(rchar, rchar+1);
	}
	
	return rstring;
}

function DOAP_call( func, params )
{
	var callurl = this.url + "?f=" + func;
	
	if (!params) params = Array();
	else if (!params instanceof Array) params = Array(params);
	
	for (var i=0; i<params.length; i++)
	{
		callurl = callurl + "&p["+i+"]="+encodeURIComponent(params[i]);
	}
	
	if (this.nocache)
	{
		callurl += ("&nc=" + this.makerandomstring());
	}
	
	if (this.xmlhttp == null) this.makexmlhttp();
	
	
	if ( this.async && (this.callback != null) ) // async - CAREFUL!
	{
		this.xmlhttp.doap = this;
		this.xmlhttp.open('GET', callurl, true);
		
		this.xmlhttp.onreadystatechange = function()
		{
			if (this.readyState == 4)
			{
				if (this.status == 200)
				{
					if (this.responseXML == null)
					{
						alert("null data");
						var error = new DOAP_Error(102, "XML Result Null (Blank, Invalid or Corrupt)");
						throw error;
					}
					else
					{
						this.doap.callback(this.doap.process(this.responseXML));
					}
				}
				else
				{
					var error = new DOAP_Error(101, "HTTP Result Not 200 ("+this.xmlhttp.status+")");
					throw error;
				}	
			}
		}
		this.xmlhttp.send();
	}
	else // non-sync
	{	
		this.xmlhttp.open('GET', callurl, false);
		this.xmlhttp.send(null);
	
		if (this.xmlhttp.status == 200)
		{
			if (this.xmlhttp.responseXML == null)
			{
				var error = new DOAP_Error(102, "XML Result Null (Blank, Invalid or Corrupt)");
				throw error;
			}
			else return this.process(this.xmlhttp.responseXML);
		}
		else
		{
			var error = new DOAP_Error(101, "HTTP Result Not 200 ("+this.xmlhttp.status+")");
			throw error;
		}	
	}
}

function DOAP_array( arrxml )
{
	var arr = Array();
	
	for (var i=0; i<arrxml.childNodes.length; i++)
	{
		var ename = arrxml.childNodes[i].tagName;
		if (ename == "ELEMENT") arr.push(arrxml.childNodes[i].firstChild.nodeValue);
		else if (ename == "ARRAY")
		{
			arr.push(this.processarray(arrxml.childNodes[i]));
		}
		else
		{
			// unknown element type in array return
		}
	}
	
	return arr;
}

function DOAP_process( xml )
{
	if (xml.documentElement.tagName == "ERROR")
	{
		var code = xml.getElementsByTagName("CODE")[0].firstChild.nodeValue;
		var desc = xml.getElementsByTagName("DESC")[0].firstChild.nodeValue;
		var error = new DOAP_Error(code, desc);
		throw error;
	}
	
	else if (xml.documentElement.tagName == "RESULT")
	{
		var result = xml.getElementsByTagName("RESULT")[0];
		if (result==null)
		{
			var error = new DOAP_Error(105,"Result Block not in XML");
			throw error;
		}
		var data = result.firstChild.nodeValue;
	}
	
	else if (xml.documentElement.tagName == "ARRAY")
	{
		var arrdata = xml.getElementsByTagName("ARRAY")[0];
		var data = this.processarray(arrdata);
	}
	
	else
	{
		var error=new DOAP_Error(106,"Unknown XML Response Type: "+xml.documentElement.tagName);
		throw error;
	}

	
	return data;
}

function DOAP()
{
	this.xmlhttp = null;
	this.url = "";
	
	this.nocache = true;
	
	this.async = false;
	this.callback = null;
	
	this.makexmlhttp = DOAP_xmlhttp;
	this.makerandomstring = DOAP_randomstring;
	this.call = DOAP_call;
	this.process = DOAP_process;
	this.processarray = DOAP_array;
}